import py
from rpython.rtyper.lltypesystem import lltype, llmemory, llarena
from rpython.memory.gc.incminimark import IncrementalMiniMarkGC, WORD
from rpython.memory.gc.incminimark import GCFLAG_VISITED
from test_direct import BaseDirectGCTest

T = lltype.GcForwardReference()
T.become(lltype.GcStruct('pinning_test_struct2',
                         ('someInt', lltype.Signed)))

S = lltype.GcForwardReference()
S.become(lltype.GcStruct('pinning_test_struct1',
                         ('someInt', lltype.Signed),
                         ('next', lltype.Ptr(T)),
                         ('data', lltype.Ptr(T))))

class PinningGCTest(BaseDirectGCTest):

    def setup_method(self, meth):
        BaseDirectGCTest.setup_method(self, meth)
        max = getattr(meth, 'max_number_of_pinned_objects', 20)
        self.gc.max_number_of_pinned_objects = max
        if not hasattr(self.gc, 'minor_collection'):
            self.gc.minor_collection = self.gc._minor_collection

    def test_pin_can_move(self):
        # even a pinned object is considered to be movable. Only the caller
        # of pin() knows if it is currently movable or not.
        ptr = self.malloc(T)
        adr = llmemory.cast_ptr_to_adr(ptr)
        assert self.gc.can_move(adr)
        assert self.gc.pin(adr)
        assert self.gc.can_move(adr)

    def test_pin_twice(self):
        ptr = self.malloc(T)
        adr = llmemory.cast_ptr_to_adr(ptr)
        assert self.gc.pin(adr)
        assert not self.gc.pin(adr)

    def test_unpin_not_pinned(self):
        # this test checks a requirement of the unpin() interface
        ptr = self.malloc(S)
        py.test.raises(Exception,
            self.gc.unpin, llmemory.cast_ptr_to_adr(ptr))

    def test__is_pinned(self):
        ptr = self.malloc(T)
        adr = llmemory.cast_ptr_to_adr(ptr)
        assert not self.gc._is_pinned(adr)
        assert self.gc.pin(adr)
        assert self.gc._is_pinned(adr)
        self.gc.unpin(adr)
        assert not self.gc._is_pinned(adr)

    def test_prebuilt_not_pinnable(self):
        ptr = lltype.malloc(T, immortal=True)
        self.consider_constant(ptr)
        assert not self.gc.pin(llmemory.cast_ptr_to_adr(ptr))
        self.gc.collect()
        assert not self.gc.pin(llmemory.cast_ptr_to_adr(ptr))

    # XXX test with multiple mallocs, and only part of them is pinned

    def test_random(self):
        # scenario: create bunch of objects. randomly pin, unpin, add to
        # stackroots and remove from stackroots.
        import random

        for i in xrange(10**3):
            obj = self.malloc(T)
            obj.someInt = 100
            #
            if random.random() < 0.5:
                self.stackroots.append(obj)
                print("+stack")
            if random.random() < 0.5:
                self.gc.pin(llmemory.cast_ptr_to_adr(obj))
                print("+pin")
            self.gc.debug_gc_step(random.randint(1, 4))
            for o in self.stackroots[:]:
                assert o.someInt == 100
                o_adr = llmemory.cast_ptr_to_adr(o)
                if random.random() < 0.1 and self.gc._is_pinned(o_adr):
                    print("-pin")
                    self.gc.unpin(o_adr)
                if random.random() < 0.1:
                    print("-stack")
                    self.stackroots.remove(o)


class TestIncminimark(PinningGCTest):
    from rpython.memory.gc.incminimark import IncrementalMiniMarkGC as GCClass
    from rpython.memory.gc.incminimark import STATE_SCANNING, STATE_MARKING

    def test_try_pin_gcref_containing_type(self):
        # scenario: incminimark's object pinning can't pin objects that may
        # contain GC pointers
        obj = self.malloc(S)
        assert not self.gc.pin(llmemory.cast_ptr_to_adr(obj))


    def test_pin_old(self):
        # scenario: try pinning an old object. This should be not possible and
        # we want to make sure everything stays as it is.
        old_ptr = self.malloc(S)
        old_ptr.someInt = 900
        self.stackroots.append(old_ptr)
        assert self.stackroots[0] == old_ptr # test assumption
        self.gc.collect()
        old_ptr = self.stackroots[0]
        # now we try to pin it
        old_adr = llmemory.cast_ptr_to_adr(old_ptr)
        assert not self.gc.is_in_nursery(old_adr)
        assert not self.gc.pin(old_adr)
        assert self.gc.pinned_objects_in_nursery == 0

    
    def pin_pin_pinned_object_count(self, collect_func):
        # scenario: pin two objects that are referenced from stackroots. Check
        # if the pinned objects count is correct, even after an other collection
        pinned1_ptr = self.malloc(T)
        pinned1_ptr.someInt = 100
        self.stackroots.append(pinned1_ptr)
        #
        pinned2_ptr = self.malloc(T)
        pinned2_ptr.someInt = 200
        self.stackroots.append(pinned2_ptr)
        #
        assert self.gc.pin(llmemory.cast_ptr_to_adr(pinned1_ptr))
        assert self.gc.pinned_objects_in_nursery == 1
        assert self.gc.pin(llmemory.cast_ptr_to_adr(pinned2_ptr))
        assert self.gc.pinned_objects_in_nursery == 2
        #
        collect_func()
        #
        assert self.gc.pinned_objects_in_nursery == 2

    def test_pin_pin_pinned_object_count_minor_collection(self):
        self.pin_pin_pinned_object_count(self.gc.minor_collection)

    def test_pin_pin_pinned_object_count_major_collection(self):
        self.pin_pin_pinned_object_count(self.gc.collect)


    def pin_unpin_pinned_object_count(self, collect_func):
        # scenario: pin an object and check the pinned object count. Unpin it
        # and check the count again.
        pinned_ptr = self.malloc(T)
        pinned_ptr.someInt = 100
        self.stackroots.append(pinned_ptr)
        pinned_adr = llmemory.cast_ptr_to_adr(pinned_ptr)
        #
        assert self.gc.pinned_objects_in_nursery == 0
        assert self.gc.pin(pinned_adr)
        assert self.gc.pinned_objects_in_nursery == 1
        collect_func()
        assert self.gc.pinned_objects_in_nursery == 1
        self.gc.unpin(pinned_adr)
        assert self.gc.pinned_objects_in_nursery == 0
        collect_func()
        assert self.gc.pinned_objects_in_nursery == 0

    def test_pin_unpin_pinned_object_count_minor_collection(self):
        self.pin_unpin_pinned_object_count(self.gc.minor_collection)

    def test_pin_unpin_pinned_object_count_major_collection(self):
        self.pin_unpin_pinned_object_count(self.gc.collect)


    def pinned_obj_in_stackroot(self, collect_func):
        # scenario: a pinned object that is part of the stack roots. Check if
        # it is not moved
        #
        ptr = self.malloc(T)
        ptr.someInt = 100
        self.stackroots.append(ptr)
        assert self.stackroots[0] == ptr # validate our assumption
        
        adr = llmemory.cast_ptr_to_adr(ptr)
        assert self.gc.is_in_nursery(adr) # to be sure
        assert self.gc.pin(adr)
        #
        # the object shouldn't move from now on
        collect_func()
        #
        # check if it is still at the same location as expected
        adr_after_collect = llmemory.cast_ptr_to_adr(self.stackroots[0])
        assert self.gc.is_in_nursery(adr_after_collect)
        assert adr == adr_after_collect
        assert self.gc._is_pinned(adr)
        assert ptr.someInt == 100
        assert self.gc.pinned_objects_in_nursery == 1

    def test_pinned_obj_in_stackroot_minor_collection(self):
        self.pinned_obj_in_stackroot(self.gc.minor_collection)

    def test_pinned_obj_in_stackroot_full_major_collection(self):
        self.pinned_obj_in_stackroot(self.gc.collect)

    def test_pinned_obj_in_stackroots_stepwise_major_collection(self):
        # scenario: same as for 'pinned_obj_in_stackroot' with minor change
        # that we do stepwise major collection and check in each step for
        # a correct state
        #
        ptr = self.malloc(T)
        ptr.someInt = 100
        self.stackroots.append(ptr)
        assert self.stackroots[0] == ptr # validate our assumption

        adr = llmemory.cast_ptr_to_adr(ptr)
        assert self.gc.is_in_nursery(adr)
        assert self.gc.pin(adr)
        #
        # the object shouldn't move from now on. Do a full round of major
        # steps and check each time for correct state
        #
        # check that we start at the expected point
        assert self.gc.gc_state == self.STATE_SCANNING
        done = False
        while not done:
            self.gc.debug_gc_step()
            # check that the pinned object didn't move
            ptr_after_collection = self.stackroots[0]
            adr_after_collection = llmemory.cast_ptr_to_adr(ptr_after_collection)
            assert self.gc.is_in_nursery(adr_after_collection)
            assert adr == adr_after_collection
            assert self.gc._is_pinned(adr)
            assert ptr.someInt == 100
            assert self.gc.pinned_objects_in_nursery == 1
            # as the object is referenced from the stackroots, the gc internal
            # 'old_objects_pointing_to_pinned' should be empty
            assert not self.gc.old_objects_pointing_to_pinned.non_empty()
            #
            # break condition
            done = self.gc.gc_state == self.STATE_SCANNING

    
    def pin_unpin_moved_stackroot(self, collect_func):
        # scenario: test if the pinned object is moved after being unpinned.
        # the second part of the scenario is the tested one. The first part
        # is already tests by other tests.
        ptr = self.malloc(T)
        ptr.someInt = 100
        self.stackroots.append(ptr)
        assert self.stackroots[0] == ptr # validate our assumption

        adr = llmemory.cast_ptr_to_adr(ptr)
        assert self.gc.pin(adr)

        collect_func()
        #
        # from here on the test really starts. previouse logic is already tested
        #
        self.gc.unpin(adr)
        assert not self.gc._is_pinned(adr)
        assert self.gc.is_in_nursery(adr)
        #
        # now we do another collection and the object should be moved out of
        # the nursery.
        collect_func()
        new_adr = llmemory.cast_ptr_to_adr(self.stackroots[0])
        assert not self.gc.is_in_nursery(new_adr)
        assert self.stackroots[0].someInt == 100
        with py.test.raises(RuntimeError) as exinfo:
            ptr.someInt = 200
        assert "freed" in str(exinfo.value)

    def test_pin_unpin_moved_stackroot_minor_collection(self):
        self.pin_unpin_moved_stackroot(self.gc.minor_collection)

    def test_pin_unpin_moved_stackroot_major_collection(self):
        self.pin_unpin_moved_stackroot(self.gc.collect)

    
    def pin_referenced_from_old(self, collect_func):
        # scenario: an old object points to a pinned one. Check if the pinned
        # object is correctly kept in the nursery and not moved.
        #
        # create old object
        old_ptr = self.malloc(S)
        old_ptr.someInt = 900
        self.stackroots.append(old_ptr)
        assert self.stackroots[0] == old_ptr # validate our assumption
        collect_func() # make it old: move it out of the nursery
        old_ptr = self.stackroots[0]
        assert not self.gc.is_in_nursery(llmemory.cast_ptr_to_adr(old_ptr))
        #
        # create young pinned one and let the old one reference the young one
        pinned_ptr = self.malloc(T)
        pinned_ptr.someInt = 100
        self.write(old_ptr, 'next', pinned_ptr)
        pinned_adr = llmemory.cast_ptr_to_adr(pinned_ptr)
        assert self.gc.pin(pinned_adr)
        assert self.gc.is_in_nursery(pinned_adr)
        assert old_ptr.next.someInt == 100
        assert self.gc.pinned_objects_in_nursery == 1
        #
        # do a collection run and make sure the pinned one didn't move
        collect_func()
        assert old_ptr.next.someInt == pinned_ptr.someInt == 100
        assert llmemory.cast_ptr_to_adr(old_ptr.next) == pinned_adr
        assert self.gc.is_in_nursery(pinned_adr)
        
    def test_pin_referenced_from_old_minor_collection(self):
        self.pin_referenced_from_old(self.gc.minor_collection)

    def test_pin_referenced_from_old_major_collection(self):
        self.pin_referenced_from_old(self.gc.collect)

    def test_pin_referenced_from_old_stepwise_major_collection(self):
        # scenario: same as in 'pin_referenced_from_old'. However,
        # this time we do a major collection step by step and check
        # between steps that the states are as expected.
        #
        # create old object
        old_ptr = self.malloc(S)
        old_ptr.someInt = 900
        self.stackroots.append(old_ptr)
        assert self.stackroots[0] == old_ptr # validate our assumption
        self.gc.minor_collection() # make it old: move it out of the nursery
        old_ptr = self.stackroots[0]
        old_adr = llmemory.cast_ptr_to_adr(old_ptr)
        assert not self.gc.is_in_nursery(old_adr)
        #
        # create young pinned one and let the old one reference the young one
        pinned_ptr = self.malloc(T)
        pinned_ptr.someInt = 100
        self.write(old_ptr, 'next', pinned_ptr)
        pinned_adr = llmemory.cast_ptr_to_adr(pinned_ptr)
        assert self.gc.pin(pinned_adr)
        assert self.gc.is_in_nursery(pinned_adr)
        assert old_ptr.next.someInt == 100
        assert self.gc.pinned_objects_in_nursery == 1
        #
        # stepwise major collection with validation between steps
        # check that we start at the expected point
        assert self.gc.gc_state == self.STATE_SCANNING
        done = False
        while not done:
            self.gc.debug_gc_step()
            #
            # make sure pinned object didn't move
            assert old_ptr.next.someInt == pinned_ptr.someInt == 100
            assert llmemory.cast_ptr_to_adr(old_ptr.next) == pinned_adr
            assert self.gc.is_in_nursery(pinned_adr)
            assert self.gc.pinned_objects_in_nursery == 1
            #
            # validate that the old object is part of the internal list
            # 'old_objects_pointing_to_pinned' as expected.
            should_be_old_adr = self.gc.old_objects_pointing_to_pinned.pop()
            assert should_be_old_adr == old_adr
            self.gc.old_objects_pointing_to_pinned.append(should_be_old_adr)
            #
            # break condition
            done = self.gc.gc_state == self.STATE_SCANNING

    
    def pin_referenced_from_old_remove_ref(self, collect_func):
        # scenario: an old object points to a pinned one. We remove the
        # reference from the old one. So nothing points to the pinned object.
        # After this the pinned object should be collected (it's dead).
        #
        # Create the objects and get them to our initial state (this is not
        # tested here, should be already tested by other tests)
        old_ptr = self.malloc(S)
        old_ptr.someInt = 900
        self.stackroots.append(old_ptr)
        assert self.stackroots[0] == old_ptr # check assumption
        collect_func() # make it old
        old_ptr = self.stackroots[0]
        #
        pinned_ptr = self.malloc(T)
        pinned_ptr.someInt = 100
        self.write(old_ptr, 'next', pinned_ptr)
        pinned_adr = llmemory.cast_ptr_to_adr(pinned_ptr)
        assert self.gc.pin(pinned_adr)
        #
        collect_func()
        # from here on we have our initial state for this test.
        #
        # first check some basic assumptions.
        assert self.gc.is_in_nursery(pinned_adr)
        assert self.gc._is_pinned(pinned_adr)
        # remove the reference
        self.write(old_ptr, 'next', lltype.nullptr(T))
        # from now on the pinned object is dead. Do a collection and make sure
        # old object still there and the pinned one is gone.
        collect_func()
        assert self.stackroots[0].someInt == 900
        assert not self.gc.old_objects_pointing_to_pinned.non_empty()
        with py.test.raises(RuntimeError) as exinfo:
            pinned_ptr.someInt = 200
        assert "freed" in str(exinfo.value)

    def test_pin_referenced_from_old_remove_ref_minor_collection(self):
        self.pin_referenced_from_old_remove_ref(self.gc.minor_collection)

    def test_pin_referenced_from_old_remove_ref_major_collection(self):
        self.pin_referenced_from_old_remove_ref(self.gc.collect)


    def pin_referenced_from_old_remove_old(self, collect_func):
        # scenario: an old object referenced a pinned object. After removing
        # the stackroot reference to the old object, bot objects (old and pinned)
        # must be collected.
        # This test is important as we expect not reachable pinned objects to
        # be collected. At the same time we have an internal list of objects
        # pointing to pinned ones and we must make sure that because of it the
        # old/pinned object survive.
        #
        # create the objects and get them to the initial state for this test.
        # Everything on the way to the initial state should be covered by
        # other tests.
        old_ptr = self.malloc(S)
        old_ptr.someInt = 900
        self.stackroots.append(old_ptr)
        collect_func()
        old_ptr = self.stackroots[0]
        #
        pinned_ptr = self.malloc(T)
        pinned_ptr.someInt = 100
        self.write(old_ptr, 'next', pinned_ptr)
        assert self.gc.pin(llmemory.cast_ptr_to_adr(pinned_ptr))
        #
        collect_func()
        #
        # now we have our initial state: old object referenced from stackroots.
        # Old object referencing a young pinned one. Next step is to make some
        # basic checks that we got the expected state.
        assert not self.gc.is_in_nursery(llmemory.cast_ptr_to_adr(old_ptr))
        assert self.gc.is_in_nursery(llmemory.cast_ptr_to_adr(pinned_ptr))
        assert pinned_ptr == old_ptr.next
        #
        # now we remove the old object from the stackroots...
        self.stackroots.remove(old_ptr)
        # ... and do a major collection (otherwise the old object wouldn't be
        # gone).
        self.gc.collect()
        # check that both objects are gone
        assert not self.gc.old_objects_pointing_to_pinned.non_empty()
        with py.test.raises(RuntimeError) as exinfo_old:
            old_ptr.someInt = 800
        assert "freed" in str(exinfo_old.value)
        #
        with py.test.raises(RuntimeError) as exinfo_pinned:
            pinned_ptr.someInt = 200
        assert "freed" in str(exinfo_pinned.value)

    def test_pin_referenced_from_old_remove_old_minor_collection(self):
        self.pin_referenced_from_old_remove_old(self.gc.minor_collection)

    def test_pin_referenced_from_old_remove_old_major_collection(self):
        self.pin_referenced_from_old_remove_old(self.gc.collect)


    def pin_referenced_from_young_in_stackroots(self, collect_func):
        # scenario: a young object is referenced from the stackroots. This
        # young object points to a young pinned object. We check if everything
        # behaves as expected after a collection: the young object is moved out
        # of the nursery while the pinned one stays where it is.
        #
        root_ptr = self.malloc(S)
        root_ptr.someInt = 900
        self.stackroots.append(root_ptr)
        assert self.stackroots[0] == root_ptr # validate assumption
        #
        pinned_ptr = self.malloc(T)
        pinned_ptr.someInt = 100
        self.write(root_ptr, 'next', pinned_ptr)
        pinned_adr = llmemory.cast_ptr_to_adr(pinned_ptr)
        assert self.gc.pin(pinned_adr)
        # check both are in nursery
        assert self.gc.is_in_nursery(llmemory.cast_ptr_to_adr(root_ptr))
        assert self.gc.is_in_nursery(pinned_adr)
        #
        # no old object yet pointing to a pinned one
        assert not self.gc.old_objects_pointing_to_pinned.non_empty()
        #
        # now we do a collection and check if the result is as expected
        collect_func()
        #
        # check if objects are where we expect them
        root_ptr = self.stackroots[0]
        assert not self.gc.is_in_nursery(llmemory.cast_ptr_to_adr(root_ptr))
        assert self.gc.is_in_nursery(pinned_adr)
        # and as 'root_ptr' object is now old, it should be tracked specially
        should_be_root_adr = self.gc.old_objects_pointing_to_pinned.pop()
        assert should_be_root_adr == llmemory.cast_ptr_to_adr(root_ptr)
        self.gc.old_objects_pointing_to_pinned.append(should_be_root_adr)
        # check that old object still points to the pinned one as expected
        assert root_ptr.next == pinned_ptr

    def test_pin_referenced_from_young_in_stackroots_minor_collection(self):
        self.pin_referenced_from_young_in_stackroots(self.gc.minor_collection)

    def test_pin_referenced_from_young_in_stackroots_major_collection(self):
        self.pin_referenced_from_young_in_stackroots(self.gc.collect)


    def pin_referenced_from_prebuilt(self, collect_func):
        # scenario: a prebuilt object points to a pinned object. Check if the
        # pinned object doesn't move and is still accessible.
        #
        prebuilt_ptr = lltype.malloc(S, immortal=True)
        prebuilt_ptr.someInt = 900
        self.consider_constant(prebuilt_ptr)
        prebuilt_adr = llmemory.cast_ptr_to_adr(prebuilt_ptr)
        collect_func()
        #        
        pinned_ptr = self.malloc(T)
        pinned_ptr.someInt = 100
        self.write(prebuilt_ptr, 'next', pinned_ptr)
        pinned_adr = llmemory.cast_ptr_to_adr(pinned_ptr)
        assert self.gc.pin(pinned_adr)
        #
        # check if everything is as expected
        assert not self.gc.is_in_nursery(prebuilt_adr)
        assert self.gc.is_in_nursery(pinned_adr)
        assert pinned_ptr == prebuilt_ptr.next
        assert pinned_ptr.someInt == 100
        #
        # do a collection and check again
        collect_func()
        assert self.gc.is_in_nursery(pinned_adr)
        assert pinned_ptr == prebuilt_ptr.next
        assert pinned_ptr.someInt == 100

    def test_pin_referenced_from_prebuilt_minor_collection(self):
        self.pin_referenced_from_prebuilt(self.gc.minor_collection)

    def test_pin_referenced_from_prebuilt_major_collection(self):
        self.pin_referenced_from_prebuilt(self.gc.collect)


    def test_old_objects_pointing_to_pinned_not_exploading(self):
        # scenario: two old object, each pointing twice to a pinned object.
        # The internal 'old_objects_pointing_to_pinned' should contain
        # always two objects.
        # In previous implementation the list exploded (grew with every minor
        # collection), hence this test.
        old1_ptr = self.malloc(S)
        old1_ptr.someInt = 900
        self.stackroots.append(old1_ptr)
        
        old2_ptr = self.malloc(S)
        old2_ptr.someInt = 800
        self.stackroots.append(old2_ptr)
        
        pinned_ptr = self.malloc(T)
        pinned_ptr.someInt = 100
        assert self.gc.pin(llmemory.cast_ptr_to_adr(pinned_ptr))
        
        self.write(old1_ptr, 'next', pinned_ptr)
        self.write(old1_ptr, 'data', pinned_ptr)
        self.write(old2_ptr, 'next', pinned_ptr)
        self.write(old2_ptr, 'data', pinned_ptr)

        self.gc.collect()
        old1_ptr = self.stackroots[0]
        old2_ptr = self.stackroots[1]
        assert not self.gc.is_in_nursery(llmemory.cast_ptr_to_adr(old1_ptr))
        assert not self.gc.is_in_nursery(llmemory.cast_ptr_to_adr(old2_ptr))

        # do multiple rounds to make sure
        for _ in range(10):
            assert self.gc.old_objects_pointing_to_pinned.length() == 2
            self.gc.debug_gc_step()


    def pin_shadow_1(self, collect_func):
        ptr = self.malloc(T)
        adr = llmemory.cast_ptr_to_adr(ptr)
        self.stackroots.append(ptr)
        ptr.someInt = 100
        assert self.gc.pin(adr)
        self.gc.id(ptr) # allocate shadow
        collect_func()
        assert self.gc.is_in_nursery(adr)
        assert ptr.someInt == 100
        self.gc.unpin(adr)
        collect_func() # move to shadow
        adr = llmemory.cast_ptr_to_adr(self.stackroots[0])
        assert not self.gc.is_in_nursery(adr)

    def test_pin_shadow_1_minor_collection(self):
        self.pin_shadow_1(self.gc.minor_collection)

    def test_pin_shadow_1_major_collection(self):
        self.pin_shadow_1(self.gc.collect)


    def test_malloc_different_types(self):
        # scenario: malloc two objects of different type and pin them. Do a
        # minor and major collection in between. This test showed a bug that was
        # present in a previous implementation of pinning.
        obj1 = self.malloc(T)
        self.stackroots.append(obj1)
        assert self.gc.pin(llmemory.cast_ptr_to_adr(obj1))
        #
        self.gc.collect()
        #
        obj2 = self.malloc(T)
        self.stackroots.append(obj2)
        assert self.gc.pin(llmemory.cast_ptr_to_adr(obj2))


    def test_objects_to_trace_bug(self):
        # scenario: In a previous implementation there was a bug because of a
        # dead pointer inside 'objects_to_trace'. This was caused by the first
        # major collection step that added the pointer to the list and right
        # after the collection step the object is unpinned and freed by the minor
        # collection, leaving a dead pointer in the list.
        pinned_ptr = self.malloc(T)
        pinned_ptr.someInt = 101
        self.stackroots.append(pinned_ptr)
        pinned_adr = llmemory.cast_ptr_to_adr(pinned_ptr)
        assert self.gc.pin(pinned_adr)
        self.gc.debug_gc_step()
        self.gc.unpin(pinned_adr)
        self.gc.debug_gc_step()


    def pin_shadow_2(self, collect_func):
        ptr = self.malloc(T)
        adr = llmemory.cast_ptr_to_adr(ptr)
        self.stackroots.append(ptr)
        ptr.someInt = 100
        assert self.gc.pin(adr)
        self.gc.identityhash(ptr) # allocate shadow
        collect_func()
        assert self.gc.is_in_nursery(adr)
        assert ptr.someInt == 100
        self.gc.unpin(adr)
        collect_func() # move to shadow
        adr = llmemory.cast_ptr_to_adr(self.stackroots[0])
        assert not self.gc.is_in_nursery(adr)

    def test_pin_shadow_2_minor_collection(self):
        self.pin_shadow_2(self.gc.minor_collection)

    def test_pin_shadow_2_major_collection(self):
        self.pin_shadow_2(self.gc.collect)


    def test_pin_nursery_top_scenario1(self):
        ptr1 = self.malloc(T)
        adr1 = llmemory.cast_ptr_to_adr(ptr1)
        ptr1.someInt = 101
        self.stackroots.append(ptr1)
        assert self.gc.pin(adr1)
        
        ptr2 = self.malloc(T)
        adr2 = llmemory.cast_ptr_to_adr(ptr2)
        ptr2.someInt = 102
        self.stackroots.append(ptr2)
        assert self.gc.pin(adr2)

        ptr3 = self.malloc(T)
        adr3 = llmemory.cast_ptr_to_adr(ptr3)
        ptr3.someInt = 103
        self.stackroots.append(ptr3)
        assert self.gc.pin(adr3)

        # scenario: no minor collection happened, only three mallocs
        # and pins
        #
        # +- nursery
        # |
        # v
        # +--------+--------+--------+---------------------...---+
        # | pinned | pinned | pinned | empty                     |
        # +--------+--------+--------+---------------------...---+
        #                            ^                           ^
        #                            |                           |
        #              nursery_free -+                           |
        #                                           nursery_top -+
        #
        assert adr3 < self.gc.nursery_free
        assert self.gc.nursery_free < self.gc.nursery_top


    def test_pin_nursery_top_scenario2(self):
        ptr1 = self.malloc(T)
        adr1 = llmemory.cast_ptr_to_adr(ptr1)
        ptr1.someInt = 101
        self.stackroots.append(ptr1)
        assert self.gc.pin(adr1)
        
        ptr2 = self.malloc(T)
        adr2 = llmemory.cast_ptr_to_adr(ptr2)
        ptr2.someInt = 102
        self.stackroots.append(ptr2)
        assert self.gc.pin(adr2)

        ptr3 = self.malloc(T)
        adr3 = llmemory.cast_ptr_to_adr(ptr3)
        ptr3.someInt = 103
        self.stackroots.append(ptr3)
        assert self.gc.pin(adr3)

        # scenario: after first GC minor collection
        #
        # +- nursery
        # |
        # v
        # +--------+--------+--------+---------------------...---+
        # | pinned | pinned | pinned | empty                     |
        # +--------+--------+--------+---------------------...---+
        # ^
        # |
        # +- nursery_free
        # +- nursery_top
        #
        self.gc.collect()

        assert self.gc.nursery_free == self.gc.nursery_top
        assert self.gc.nursery_top == self.gc.nursery
        assert self.gc.nursery_top < adr3


    def test_pin_nursery_top_scenario3(self):
        ptr1 = self.malloc(T)
        adr1 = llmemory.cast_ptr_to_adr(ptr1)
        ptr1.someInt = 101
        self.stackroots.append(ptr1)
        assert self.gc.pin(adr1)
        
        ptr2 = self.malloc(T)
        adr2 = llmemory.cast_ptr_to_adr(ptr2)
        ptr2.someInt = 102
        self.stackroots.append(ptr2)
        assert self.gc.pin(adr2)

        ptr3 = self.malloc(T)
        adr3 = llmemory.cast_ptr_to_adr(ptr3)
        ptr3.someInt = 103
        self.stackroots.append(ptr3)
        assert self.gc.pin(adr3)

        # scenario: after unpinning first object and a minor
        # collection
        #
        # +- nursery
        # |
        # v
        # +--------+--------+--------+---------------------...---+
        # | empty  | pinned | pinned | empty                     |
        # +--------+--------+--------+---------------------...---+
        # ^        ^
        # |        |
        # |        +- nursery_top
        # +- nursery_free
        #
        self.gc.unpin(adr1)
        self.gc.collect()

        assert self.gc.nursery_free == self.gc.nursery
        assert self.gc.nursery_top > self.gc.nursery_free
        assert self.gc.nursery_top < adr2


    def test_pin_nursery_top_scenario4(self):
        ptr1 = self.malloc(T)
        adr1 = llmemory.cast_ptr_to_adr(ptr1)
        ptr1.someInt = 101
        self.stackroots.append(ptr1)
        assert self.gc.pin(adr1)
        
        ptr2 = self.malloc(T)
        adr2 = llmemory.cast_ptr_to_adr(ptr2)
        ptr2.someInt = 102
        self.stackroots.append(ptr2)
        assert self.gc.pin(adr2)

        ptr3 = self.malloc(T)
        adr3 = llmemory.cast_ptr_to_adr(ptr3)
        ptr3.someInt = 103
        self.stackroots.append(ptr3)
        assert self.gc.pin(adr3)

        # scenario: after unpinning first & second object and a minor
        # collection
        #
        # +- nursery
        # |
        # v
        # +-----------------+--------+---------------------...---+
        # | empty           | pinned | empty                     |
        # +-----------------+--------+---------------------...---+
        # ^                 ^
        # |                 |
        # |                 +- nursery_top
        # +- nursery_free
        #
        self.gc.unpin(adr1)
        self.gc.unpin(adr2)
        self.gc.collect()

        assert self.gc.nursery_free == self.gc.nursery
        assert self.gc.nursery_free < self.gc.nursery_top
        assert self.gc.nursery_top < adr3
        

    def test_pin_nursery_top_scenario5(self):
        ptr1 = self.malloc(T)
        adr1 = llmemory.cast_ptr_to_adr(ptr1)
        ptr1.someInt = 101
        self.stackroots.append(ptr1)
        assert self.gc.pin(adr1)
        
        ptr2 = self.malloc(T)
        adr2 = llmemory.cast_ptr_to_adr(ptr2)
        ptr2.someInt = 102
        self.stackroots.append(ptr2)
        assert self.gc.pin(adr2)

        ptr3 = self.malloc(T)
        adr3 = llmemory.cast_ptr_to_adr(ptr3)
        ptr3.someInt = 103
        self.stackroots.append(ptr3)
        assert self.gc.pin(adr3)

        # scenario: no minor collection happened, only three mallocs
        # and pins
        #
        # +- nursery
        # |
        # v
        # +--------+--------+--------+---------------------...---+
        # | pinned | pinned | pinned | empty                     |
        # +--------+--------+--------+---------------------...---+
        #                            ^                           ^
        #                            |                           |
        #              nursery_free -+                           |
        #                                           nursery_top -+
        #
        assert adr3 < self.gc.nursery_free
        assert self.gc.nursery_free < self.gc.nursery_top

        # scenario: unpin everything and minor collection
        #
        # +- nursery
        # |
        # v
        # +----------------------------------+-------------...---+
        # | reset arena                      | empty (not reset) |
        # +----------------------------------+-------------...---+
        # ^                                  ^
        # |                                  |
        # +- nursery_free                    |
        #                       nursery_top -+
        #
        self.gc.unpin(adr1)
        self.gc.unpin(adr2)
        self.gc.unpin(adr3)
        self.gc.collect()

        assert self.gc.nursery_free == self.gc.nursery
        assert self.gc.nursery_top > self.gc.nursery_free


    def fill_nursery_with_pinned_objects(self):
        typeid = self.get_type_id(T)
        size = self.gc.fixed_size(typeid) + self.gc.gcheaderbuilder.size_gc_header
        raw_size = llmemory.raw_malloc_usage(size)
        object_mallocs = self.gc.nursery_size // raw_size
        for instance_nr in xrange(object_mallocs):
            ptr = self.malloc(T)
            adr = llmemory.cast_ptr_to_adr(ptr)
            ptr.someInt = 100 + instance_nr
            self.stackroots.append(ptr)
            self.gc.pin(adr)

    def test_full_pinned_nursery_pin_fail(self):
        self.fill_nursery_with_pinned_objects()
        # nursery should be full now, at least no space for another `T`.
        # Next malloc should fail.
        py.test.raises(Exception, self.malloc, T)

    def test_full_pinned_nursery_arena_reset(self):
        # there were some bugs regarding the 'arena_reset()' calls at
        # the end of the minor collection.  This test brought them to light.
        self.fill_nursery_with_pinned_objects()
        self.gc.collect()

    def test_pinning_limit(self):
        assert self.gc.max_number_of_pinned_objects == 5
        for instance_nr in xrange(self.gc.max_number_of_pinned_objects):
            ptr = self.malloc(T)
            adr = llmemory.cast_ptr_to_adr(ptr)
            ptr.someInt = 100 + instance_nr
            self.stackroots.append(ptr)
            assert self.gc.pin(adr)
        #
        # now we reached the maximum amount of pinned objects
        ptr = self.malloc(T)
        adr = llmemory.cast_ptr_to_adr(ptr)
        self.stackroots.append(ptr)
        assert not self.gc.pin(adr)
    test_pinning_limit.max_number_of_pinned_objects = 5

    def test_full_pinned_nursery_pin_fail(self):
        typeid = self.get_type_id(T)
        size = self.gc.fixed_size(typeid) + self.gc.gcheaderbuilder.size_gc_header
        raw_size = llmemory.raw_malloc_usage(size)
        object_mallocs = self.gc.nursery_size // raw_size
        # just to be sure we do not run into the limit as we test not the limiter
        # but rather the case of a nursery full with pinned objects.
        assert object_mallocs < self.gc.max_number_of_pinned_objects
        for instance_nr in xrange(object_mallocs):
            ptr = self.malloc(T)
            adr = llmemory.cast_ptr_to_adr(ptr)
            ptr.someInt = 100 + instance_nr
            self.stackroots.append(ptr)
            self.gc.pin(adr)
        #
        # nursery should be full now, at least no space for another `T`.
        # Next malloc should fail.
        py.test.raises(Exception, self.malloc, T)
    test_full_pinned_nursery_pin_fail.max_number_of_pinned_objects = 50


    def test_pin_bug1(self):
        #
        # * the nursery contains a pinned object 'ptr1'
        #
        # * outside the nursery is another object 'ptr2' pointing to 'ptr1'
        #
        # * during one incremental tracing step, we see 'ptr2' but don't
        #   trace 'ptr1' right now: it is left behind on the trace-me-later
        #   list
        #
        # * then we run the program, unpin 'ptr1', and remove it from 'ptr2'
        #
        # * at the next minor collection, we free 'ptr1' because we don't
        #   find anything pointing to it (it is removed from 'ptr2'),
        #   but 'ptr1' is still in the trace-me-later list
        #
        # * the trace-me-later list is deep enough that 'ptr1' is not
        #   seen right now!  it is only seen at some later minor collection
        #
        # * at that later point, crash, because 'ptr1' in the nursery was
        #   overwritten
        #
        ptr2 = self.malloc(S)
        ptr2.someInt = 102
        self.stackroots.append(ptr2)

        self.gc.collect()
        ptr2 = self.stackroots[-1]    # now outside the nursery
        adr2 = llmemory.cast_ptr_to_adr(ptr2)

        ptr1 = self.malloc(T)
        adr1 = llmemory.cast_ptr_to_adr(ptr1)
        ptr1.someInt = 101
        self.write(ptr2, 'data', ptr1)
        res = self.gc.pin(adr1)
        assert res

        self.gc.minor_collection()
        assert self.gc.gc_state == self.STATE_SCANNING
        self.gc.major_collection_step()
        assert self.gc.objects_to_trace.tolist() == [adr2]
        assert self.gc.more_objects_to_trace.tolist() == []

        self.gc.TEST_VISIT_SINGLE_STEP = True

        self.gc.minor_collection()
        assert self.gc.gc_state == self.STATE_MARKING
        self.gc.major_collection_step()
        assert self.gc.objects_to_trace.tolist() == []
        assert self.gc.more_objects_to_trace.tolist() == [adr2]

        self.write(ptr2, 'data', lltype.nullptr(T))
        self.gc.unpin(adr1)

        assert ptr1.someInt == 101
        self.gc.minor_collection()        # should free 'ptr1'
        py.test.raises(RuntimeError, "ptr1.someInt")
        assert self.gc.gc_state == self.STATE_MARKING
        self.gc.major_collection_step()   # should not crash reading 'ptr1'!

        del self.gc.TEST_VISIT_SINGLE_STEP


    def test_pin_bug2(self):
        #
        # * we have an old object A that points to a pinned object B
        #
        # * we unpin B
        #
        # * the next minor_collection() is done in STATE_MARKING==1
        #   when the object A is already black
        #
        # * _minor_collection() => _visit_old_objects_pointing_to_pinned()
        #   which will move the now-unpinned B out of the nursery, to B'
        #
        # At that point we need to take care of colors, otherwise we
        # get a black object (A) pointing to a white object (B'),
        # which must never occur.
        #
        ptrA = self.malloc(T)
        ptrA.someInt = 42
        adrA = llmemory.cast_ptr_to_adr(ptrA)
        res = self.gc.pin(adrA)
        assert res

        ptrC = self.malloc(S)
        self.stackroots.append(ptrC)

        ptrB = self.malloc(S)
        ptrB.data = ptrA
        self.stackroots.append(ptrB)

        self.gc.collect()
        ptrB = self.stackroots[-1]    # now old and outside the nursery
        ptrC = self.stackroots[-2]    # another random old object, traced later
        adrB = llmemory.cast_ptr_to_adr(ptrB)

        self.gc.minor_collection()
        assert self.gc.gc_state == self.STATE_SCANNING
        self.gc.major_collection_step()
        assert self.gc.gc_state == self.STATE_MARKING
        assert not (self.gc.header(adrB).tid & GCFLAG_VISITED)  # not black yet

        self.gc.TEST_VISIT_SINGLE_STEP = True
        self.gc.major_collection_step()
        assert self.gc.gc_state == self.STATE_MARKING
        assert self.gc.header(adrB).tid & GCFLAG_VISITED    # now black
        # but ptrC is not traced yet, which is why we're still in STATE_MARKING
        assert self.gc.old_objects_pointing_to_pinned.tolist() == [adrB]

        self.gc.unpin(adrA)

        self.gc.DEBUG = 2
        self.gc.minor_collection()



import sys
import opcode
import dis
import imp
import os
import __builtin__
import time

"""
so design goal:

i want to take a pile of source code and analyze each module for the
names it defines and the modules it imports and the names it uses from
them.

then i can find things like:

- things which are just plain not used anywhere
- things which are defined in one module and only used in another
- importing of names from modules where they are just imported from
  somewhere else
- cycles in the import graph
- unecessary imports

finding imports at top level is fairly easy, although the variety of
types of import statement can be baffling.  a mini reference:

import foo

->

LOAD_CONST None
IMPORT_NAME foo
STORE_NAME foo


import foo as bar

->

LOAD_CONST None
IMPORT_NAME foo
STORE_NAME bar

from foo import bar

->

LOAD_CONST  ('bar',)
IMPORT_NAME foo
IMPORT_FROM bar
STORE_NAME  bar
POP_TOP

from foo import bar, baz

->

LOAD_CONST  ('bar','baz')
IMPORT_NAME foo
IMPORT_FROM bar
STORE_NAME  bar
IMPORT_FROM baz
STORE_NAME  baz
POP_TOP

from foo.baz import bar

->

LOAD_CONST  ('bar',)
IMPORT_NAME foo.baz
IMPORT_FROM bar
STORE_NAME  bar
POP_TOP


import foo.bar

->

LOAD_CONST  None
IMPORT_NAME foo.bar
STORE_NAME  foo

(I hate this style)

there are other forms, but i don't support them (should hit an
assertion rather than silently fail).

"""

class System:
    def __init__(self):
        self.modules = {}
        self.pendingmodules = {}

class Scope(object):
    def __init__(self, parent=None):
        self.modvars = {} # varname -> absolute module name
        self.parent = parent
        self.varsources = {}

    def mod_for_name(self, name):
        if name in self.modvars:
            return self.modvars[name]
        elif self.parent is not None:
            return self.parent.mod_for_name(name)
        else:
            return None

    def var_source(self, name):
        if name in self.varsources:
            return self.varsources[name]
        elif self.parent is not None:
            return self.parent.var_source(name)
        else:
            return None, None


class Module(object):
    def __init__(self, name, system):
        self.name = name
        self.system = system
        self._imports = {} # {modname:{name:was-it-used?}}
        self.definitions = ['__file__']
        if name == 'pypy.objspace.std.objspace':
            self.definitions.extend([
                'W_NoneObject', 'W_BoolObject', 'W_BoolObject', 'W_TypeObject',
                'W_TypeObject', 'W_TypeObject', 'W_IntObject',
                'W_StringObject', 'W_UnicodeObject', 'W_FloatObject',
                'W_TupleObject', 'W_ListObject', 'W_LongObject', 'W_SliceObject',
                'W_IntObject', 'W_FloatObject', 'W_LongObject', 'W_TupleObject',
                'W_ListObject', 'W_DictObject', 'W_SliceObject',
                'W_StringObject', 'W_UnicodeObject', 'W_SeqIterObject',
                'W_TupleObject', 'W_DictObject', 'W_DictObject'])
        self.toplevelscope = Scope()
        self.importers = []
    def import_(self, modname):
        # should probably handle relative imports here.
        if modname not in self._imports:
            if recursive and modname not in self.system.modules:
                self.system.pendingmodules[modname] = None
            self._imports[modname] = {}
        return self._imports[modname]

def iteropcodes(codestring):
    n = len(codestring)
    i = 0
    while i < n:
        op = ord(codestring[i])
        i += 1
        oparg = None
        assert op != opcode.EXTENDED_ARG, 'EXTENDED_ARG'
        if op >= opcode.HAVE_ARGUMENT:
            oparg = ord(codestring[i]) + ord(codestring[i+1])*256
            i += 2
        yield op, oparg


class _Op(object):
    def __getattr__(self, name):
        if name in opcode.opmap:
            return opcode.opmap[name]
        else:
            raise AttributeError(name)

_op_ = _Op()


loadops = [_op_.LOAD_NAME, _op_.LOAD_GLOBAL, _op_.LOAD_FAST, _op_.LOAD_DEREF]
storeops = [_op_.STORE_NAME, _op_.STORE_FAST, _op_.STORE_DEREF, _op_.STORE_GLOBAL]

def name_for_op(code, op, oparg):
    if op in [_op_.LOAD_GLOBAL, _op_.STORE_GLOBAL, _op_.LOAD_NAME,  _op_.STORE_NAME]:
        return code.co_names[oparg]
    elif op in [_op_.LOAD_FAST, _op_.STORE_FAST]:
        return code.co_varnames[oparg]
    elif op in [_op_.LOAD_DEREF, _op_.STORE_DEREF]:
        if oparg < len(code.co_cellvars):
            return code.co_cellvars[oparg]
        else:
            return code.co_freevars[oparg - len(code.co_cellvars)]
    else:
        assert 0, "%s is not an opcode with a name!"%(opcode.opname[op],)

def process(r, codeob, scope, toplevel=False):
    opcodes = list(iteropcodes(codeob.co_code))

    i = 0

    codeobjs = []

    while i < len(opcodes):
        op, oparg = opcodes[i]

        if op == _op_.IMPORT_NAME:
            preop, preoparg = opcodes[i-1]
            assert preop == _op_.LOAD_CONST, 'LOAD_CONST'

            fromlist = codeob.co_consts[preoparg]

            modname = codeob.co_names[oparg]

            if fromlist is None:
                # this is the 'import foo' case
                r.import_(modname)

                seenloadattr = False
                while 1:
                    postop, postoparg = opcodes[i+1]
                    i += 1
                    if postop != _op_.LOAD_ATTR:
                        break
                    seenloadattr = True
                    
                assert postop in storeops, 'postop'

                storename = name_for_op(codeob, postop, postoparg)
                if seenloadattr:
                    scope.modvars[storename] = modname
                else:
                    scope.modvars[storename] = modname.split('.')[0]
            elif fromlist == ('*',):
                assert toplevel, 'toplevel'
                if modname.startswith('pypy.'):
                    if modname not in r.system.modules:
                        if modname in r.system.pendingmodules:
                            del r.system.pendingmodules[modname]
                        process_module(modname, r.system)
                    M = r.system.modules[modname]
                    for d in M.definitions + list(M.toplevelscope.modvars) + \
                            [a[1] for a in M.toplevelscope.varsources.itervalues()]:
                        if d[0] != '_':
                            #print '* got ', d
                            scope.varsources[d] = modname, d
                            r.import_(modname)[d] = -1
                r.import_(modname)['*'] = True
            else:
                # ok, this is from foo import bar
                path = None
                try:
                    for part in modname.split('.'):
                        path = [imp.find_module(part, path)[1]]
                except ImportError:
                    path = -1
                i += 1
                for f in fromlist:
                    op, oparg = opcodes[i]
                    assert op == _op_.IMPORT_FROM, 'IMPORT_FROM'
                    assert codeob.co_names[oparg] == f, 'f'
                    i += 1

                    if path == -1:
                        i += 1
                        continue

                    op, oparg = opcodes[i]
                    i += 1

                    assert op in storeops, 'opstore'
                    storename = name_for_op(codeob, op, oparg)

                    try:
                        imp.find_module(f, path)
                    except ImportError:
                        r.import_(modname)[f] = False
                        scope.varsources[storename] = modname, f
                    else:
                        submod = modname + '.' + f
                        r.import_(submod)
                        scope.modvars[storename] = submod

                op, oparg = opcodes[i]
                assert op == _op_.POP_TOP, 'POP_TOP'
        elif op == _op_.STORE_NAME and toplevel or op == _op_.STORE_GLOBAL:
            r.definitions.append(codeob.co_names[oparg])
        elif op == _op_.LOAD_ATTR:
            preop, preoparg = opcodes[i-1]
            if preop in loadops:
                name = name_for_op(codeob, preop, preoparg)
                m = scope.mod_for_name(name)
                if m:
                    r.import_(m)[codeob.co_names[oparg]] = True
        elif op in loadops:
            name = name_for_op(codeob, op, oparg)
            m, a = scope.var_source(name)
            if m:
                assert a in r.import_(m), 'a'
                r.import_(m)[a] = True
##             else:
##                 if name not in r.definitions \
##                        and scope.mod_for_name(name) is None \
##                        and scope.var_source(name) == (None, None) \
##                        and name not in __builtin__.__dict__ \
##                        and (op == LOAD_GLOBAL or toplevel):
##                     print 'where did', name, 'come from?'
        elif op in [_op_.MAKE_FUNCTION, _op_.MAKE_CLOSURE]:
            preop, preoparg = opcodes[i-1]
            assert preop == _op_.LOAD_CONST, 'preop'
            codeobjs.append(codeob.co_consts[preoparg])

        i += 1
    for c in codeobjs:
        process(r, c, Scope(scope))

def find_from_dotted_name(modname):
    path = None
    for part in modname.split('.'):
        try:
            path = [imp.find_module(part, path)[1]]
        except ImportError:
            print modname
            raise
    return path[0]

def process_module(dottedname, system):
    if dottedname.endswith('.py'):
        path = dottedname
        dottedname = path.lstrip('./').rstrip()[:-3].replace('/', '.')
    else:
        path = find_from_dotted_name(dottedname)

    ispackage = False
    if os.path.isdir(path):
        ispackage = True
        path += '/__init__.py'
    r = Module(dottedname, system)
    r.ispackage = ispackage

    if dottedname in system.modules:
        return system.modules[dottedname]

    try:
        code = compile(open(path, "U").read(), path, 'exec')
        process(r, code, r.toplevelscope, True)
    except (ImportError, SyntaxError) as e:
        print "failed!", e
        #raise
    else:
        if dottedname in system.pendingmodules:
            print
            del system.pendingmodules[dottedname]

        system.modules[dottedname] = r

    return r

# --- stuff that uses the processed data ---

def report_unused_symbols(system):
    for name, mod in sorted(system.modules.iteritems()):
        printed = False
        if not 'pypy.' in name or '_cache' in name:
            continue
        u = {}
        for n in mod._imports:
            if n in ('autopath', '__future__'):
                continue
            usedany = False
            for field, used in mod._imports[n].iteritems():
                if n in system.modules:
                    M = system.modules[n]
                    if not M.ispackage and field != '*' and field not in M.definitions \
                           and used != -1:
                        if not printed:
                            print '*', name
                            printed = True
                        sourcemod, nam = M.toplevelscope.var_source(field)
                        print '   ', field, 'used from', n, 'but came from', sourcemod
                if not used:
                    u.setdefault(n, []).append(field)
                else:
                    usedany = True
            if not usedany:
                if n in u:
                    u[n].append('(i.e. entirely)')
                else:
                    u[n] = 'entirely'
        if u:
            if not printed:
                print '*', name
                printed = True
            for k, v in u.iteritems():
                print '   ', k, v

def find_cycles(system):
    from rpython.tool.algo import graphlib
    vertices = dict.fromkeys(system.modules)
    edges = {}
    for m in system.modules:
        edges[m] = []
        for n in system.modules[m]._imports:
            edges[m].append(graphlib.Edge(m, n))
    cycles = []
    for component in graphlib.strong_components(vertices, edges):
        random_vertex = component.iterkeys().next()
        cycles.extend(graphlib.all_cycles(random_vertex, component, edges))

    ncycles = []
    for cycle in cycles:
        packs = {}
        for edge in cycle:
            package = edge.source.rsplit('.', 1)[0]
            packs[package] = True
        if len(packs) > 1:
            ncycles.append(cycle)
    cycles = ncycles

    for cycle in cycles:
        l = len(cycle[0].source)
        print cycle[0].source, '->', cycle[0].target
        for edge in cycle[1:]:
            print ' '*l, '->', edge.target
    print len(cycles), 'inter-package cycles'

def summary(system):
    mcount = float(len(system.modules))
    importcount = 0
    importstars = 0
    importstarusage = 0
    defcount = 0
    importedcount = 0
    for m in system.modules:
        m = system.modules[m]
        defcount += len(m.definitions)
        importedcount += len(m.importers)
        importcount += len(m._imports)
        for n in m._imports:
            if '*' in m._imports[n]:
                importstars += 1
                importstarusage += len([o for (o, v) in m._imports[n].iteritems() if v == True])
    print
    print 'the average module'
    print 'was imported %.2f times'%(importedcount/mcount)
    print 'imported %.2f other modules'%(importcount/mcount)
    print 'defined %.2f names'%(defcount/mcount)
    print
    print 'there were %d import *s'%(importstars)
    print 'the average one produced %.2f names that were actually used'\
          %((1.0*importstarusage)/importstars)

def not_imported(system):
    for m, M in sorted(system.modules.iteritems()):
        if not M.importers and 'test' not in m and '__init__' not in m:
            print m

def import_stars(system):
    for m in sorted(system.modules):
        m = system.modules[m]
        for n in sorted(m._imports):
            if '*' in m._imports[n]:
                print m.name, 'imports * from', n
                used = [o for (o, v) in m._imports[n].iteritems() if v == True and o != '*']
                print len(used), 'out of', len(m._imports[n]) - 1, 'names are used'
                print '    ', ', '.join(sorted(used))

def find_varargs_users(system):
    for m in sorted(system.modules):
        m = system.modules[m]
        if 'pypy.interpreter.pycode' in m._imports:
            if m._imports['pypy.interpreter.pycode'].get('CO_VARARGS') == True:
                print m.name

# --- HTML generation stuff ---

def file_for_module(module):
    fname = os.path.join('importfunhtml', *module.name.split('.')) + '.html'
    dname = os.path.dirname(fname)
    if not os.path.isdir(dname):
        os.makedirs(dname)
    return open(fname, 'w')

def link_for_module(fromlink, module):
    link = '/'.join(module.name.split('.')) + '.html'
    prefix = '/'.join(['..']*fromlink.count('/'))
    if prefix:
        return prefix + '/' + link
    else:
        return link

def file_for_name(module, name):
    fname = os.path.join('importfunhtml', *(module.name.split('.') + [name])) + '.html'
    dname = os.path.dirname(fname)
    if not os.path.isdir(dname):
        os.makedirs(dname)
    return open(fname, 'w')

def link_for_name(fromlink, module, name):
    link = '/'.join(module.name.split('.') + [name]) + '.html'
    prefix = '/'.join(['..']*fromlink.count('/'))
    if prefix:
        return prefix + '/' + link
    else:
        return link

def html_for_module(module):
    from py.xml import html
    out = file_for_module(module)
    ourlink = link_for_module('', module)
    head = [html.title(module.name)]
    body = [html.h1(module.name)]
    body.append(html.p('This module defines these names:'))
    listbody = []
    defuses = {}
    for d in module.definitions:
        uses = []
        for n in sorted(module.importers):
            N = module.system.modules[n]
            if N._imports[module.name].get(d) == True:
                uses.append(n)

        if not d.startswith('_'):
            if uses:
                listbody.append(html.li(
                    html.a(d, href=link_for_name(ourlink, module, d))))
                defuses[d] = uses
            else:
                listbody.append(html.li(d))

    body.append(html.ul(listbody))
    body.append(html.p('This module imports the following:'))
    listbody1 = []
    for n in sorted(module._imports):
        if n in ('autopath', '__future__'):
            continue
        if n in module.system.modules:
            listbody2 = [html.a(
                n, href=link_for_module(ourlink, module.system.modules[n]))]
        else:
            listbody2 = [n]
        listbody3 = []
        for o in sorted(module._imports[n]):
            if module._imports[n][o] == True:
                if n in module.system.modules:
                    listbody3.append(
                        html.li(html.a(
                        o, href=link_for_name(ourlink, module.system.modules[n], o))))
                else:
                    listbody3.append(html.li(o))
        if listbody3:
            listbody2.append(html.ul(listbody3))
        listbody1.append(html.li(listbody2))
    body.append(html.ul(listbody1))
    body.append(html.p('This module is imported by the following:'))
    listbody1 = []
    for n in module.importers:
        licontents = [html.a(n, href=link_for_module(ourlink, module.system.modules[n]))]
        contents = []
        for o in sorted(module.system.modules[n]._imports[module.name]):
            contents.append(html.li(html.a(o, href=link_for_name(ourlink, module, o))))
        if contents:
            licontents.append(html.ul(contents))
        listbody1.append(html.li(licontents))
    body.append(html.ul(listbody1))

    out.write(html.html(head, body).unicode())

    for d in defuses:
        out = file_for_name(module, d)
        ourlink = link_for_name('', module, d)
        head = [html.title(module.name + '.' + d)]
        body = [html.h1([html.a(module.name, href=link_for_module(ourlink, module)), '.' + d])]
        
        contents = []

        for n in defuses[d]: 
            N = module.system.modules[n]
            contents.append(html.li(html.a(n, href=link_for_module(ourlink, N))))

        body.append(html.p('This name is used in'))
        body.append(html.ul(contents))

        out.write(html.html(head, body).unicode())

def make_html_report(system):
    if os.path.isdir('importfunhtml'):
        os.system('rm -rf importfunhtml')
    os.mkdir('importfunhtml')
    for m in system.modules.itervalues():
        html_for_module(m)

# --- the driver stuff! ---

def main(*paths):
    system = System()

    for path in paths:
        system.pendingmodules[path] = None

    T = time.time()

    while system.pendingmodules:
        path, d = system.pendingmodules.popitem()
        if sys.stdout.isatty():
            print '\r\033[K', len(system.modules), '/', len(system.pendingmodules), path,
            sys.stdout.flush()
        if '._cache' in path or '/_cache' in path:
            continue
        if '/' not in path and not path.startswith('pypy.'):
            continue
        process_module(path, system)

    print
    print 'analysed', len(system.modules), 'modules in %.2f seconds'%(time.time() - T)
    print '------'

    # record importer information
    for name, mod in system.modules.iteritems():
        for n in mod._imports:
            if n in system.modules:
                system.modules[n].importers.append(name)

    make_html_report(system)

    if interactive:
        import pdb
        pdb.set_trace()

recursive = False
interactive = False

if __name__=='__main__':
    if '-r' in sys.argv:
        recursive = True
        sys.argv.remove('-r')
    if '-i' in sys.argv:
        interactive = True
        sys.argv.remove('-i')
    if len(sys.argv) > 1:
        main(*sys.argv[1:])
    else:
        paths = []
        for line in os.popen("find pypy -name '*.py'"):
            paths.append(line[:-1])
        main(*paths)



"""Test comparisons of Exceptions in except clauses.

New for PyPy - Could be incorporated into CPython regression tests.
"""

class AppTestExceptionComp: 

    def test_exception(self):
        try:
            raise TypeError("nothing")
        except TypeError:
            pass
        except:
            self.fail("Identical exceptions do not match.") 

    def test_exceptionfail(self):
        try:
            raise TypeError("nothing")
        except KeyError:
            self.fail("Different exceptions match.")
        except TypeError:
            pass
        except:
            self.fail("Unanticipated value for exception raise.")
            

    def test_called(self):
        try:
            raise SyntaxError("Invalid")
        except SyntaxError:
            pass
        except:
            self.fail("Instantiated exception does not match parent class.") 

    def test_calledfail(self):
        try:
            raise SyntaxError("Invalid")
        except ZeroDivisionError:
            self.fail("Instantiated exception matches different parent class.") 
        except SyntaxError:
            pass
        except:
            self.fail("Unanticpated value for exception raise.")
            
        
    def test_userclass(self):
        class UserExcept(Exception):
            pass
        try:
            raise UserExcept("nothing")
        except UserExcept:
            pass
        except:
            self.fail("User defined class exceptions do not match.") 
            
    def test_subclass(self):
        try:
            raise KeyError("key")
        except LookupError:
            pass
        except:
            self.fail("Exception does not match parent class.") 

    def test_deepsubclass(self):
        try:
            raise FloatingPointError("1.2r")
        except Exception:
            pass
        except:
            self.fail("Exception does not match grandparent class.") 

    def test_tuple(self):
        try:
            raise ArithmeticError("2+jack")
        except (ZeroDivisionError, ArithmeticError):
            pass
        except:
            self.fail("Exception does not match self in tuple.") 

    def test_parenttuple(self):
        try:
            raise ZeroDivisionError("0")
        except (StandardError, SystemExit):
            pass
        except:
            self.fail("Exception does not match parent in tuple.") 

    def test_nestedtuples(self):
        try:
            raise AssertionError("0")
        except (SystemExit, (KeyboardInterrupt, AssertionError)):
            pass
        except:
            self.fail("Exception does not match self in nested tuple.") 

    def test_deeptuples(self):
        try:
            raise IOError
        except (FloatingPointError,(OSError,
                                    (SyntaxError,IOError,ZeroDivisionError)),
                (MemoryError, NotImplementedError)):
            pass
        except:
            self.fail("Exception does not match self in deeply nested tuple.")



#!/usr/bin/python
from array import array

def f(img):
    l=0
    i=0;
    while i<640*480:
        l+=img[i]
        i+=1
    return l

img=array('d', (0,)) * (640*480)
    
for l in range(500): f(img)

#           C          pypy-simple pypy        cpython
# sumtst:   0m0.630s   0m0.659s    0m0.851s    0m33.447s
# intimg:   0m0.646s   0m1.078s    0m1.446s    1m0.279s



from rpython.jit.backend.arm.test.support import skip_unless_run_slow_tests
skip_unless_run_slow_tests()

from rpython.jit.backend.llsupport.test.zrpy_gc_test import CompileFrameworkTests


class TestShadowStack(CompileFrameworkTests):
    gcrootfinder = "shadowstack"



from pypy.interpreter.mixedmodule import MixedModule
from pypy.interpreter.gateway import unwrap_spec
from pypy.interpreter.pyparser import pytoken, pygram


class Module(MixedModule):

    appleveldefs = {}
    interpleveldefs = {
        "NT_OFFSET" : "space.newint(256)",
        "ISTERMINAL" : "moduledef.isterminal",
        "ISNONTERMINAL" : "moduledef.isnonterminal",
        "ISEOF" : "moduledef.iseof"
        }


def _init_tokens():
    tok_name = {}
    for tok, id in pytoken.python_tokens.iteritems():
        Module.interpleveldefs[tok] = "space.wrap(%d)" % (id,)
        tok_name[id] = tok
    Module.interpleveldefs["tok_name"] = "space.wrap(%r)" % (tok_name,)
    Module.interpleveldefs["N_TOKENS"] = "space.wrap(%d)" % len(tok_name)

_init_tokens()


@unwrap_spec(tok=int)
def isterminal(space, tok):
    return space.newbool(tok < 256)

@unwrap_spec(tok=int)
def isnonterminal(space, tok):
    return space.newbool(tok >= 256)

@unwrap_spec(tok=int)
def iseof(space, tok):
    return space.newbool(tok == pygram.tokens.ENDMARKER)



from rpython.jit.backend.aarch64.test.test_basic import JitAarch64Mixin
from rpython.jit.metainterp.test import test_call

class TestCall(JitAarch64Mixin, test_call.CallTest):
    # for the individual tests see
    # ====> ../../../metainterp/test/test_call.py
    pass



# overrides for annotation specific to PyPy codebase
from rpython.annotator.policy import AnnotatorPolicy
from rpython.flowspace.model import Constant
from rpython.annotator.classdesc import InstanceSource

def set_attribute(classdesc, value, attr="_static_lookup_cache"):
    cls = classdesc.pyobj
    if attr in cls.__dict__:
        return
    classdesc.classdict[attr] = Constant(value)
    classdesc.immutable_fields.add(attr)
    classdesc.classdef.add_source_for_attribute(attr, classdesc)
    setattr(cls, attr, value)

class PyPyAnnotatorPolicy(AnnotatorPolicy):
    def __init__(self, space):
        self.lookups = set()
        self.lookups_where = set()
        self.pypytypes = {} # W_Root subclass -> W_TypeObject()
        self.types_w = set()
        self.space = space

    def consider_lookup(self, bookkeeper, attr):
        from pypy.objspace.std import typeobject
        from pypy.interpreter import baseobjspace
        assert attr not in self.lookups
        cached = "cached_%s" % attr
        clsdef = bookkeeper.getuniqueclassdef(StaticLookupCache)
        classdesc = clsdef.classdesc
        set_attribute(classdesc, None, cached)
        for cls, w_type in self.pypytypes.items():
            if cls._static_lookup_cache is not None:
                setattr(cls._static_lookup_cache, cached, w_type._lookup(attr))
                source = InstanceSource(bookkeeper, cls._static_lookup_cache)
                clsdef.add_source_for_attribute(cached, source)
        self.lookups.add(attr)

    def consider_lookup_in_type_where(self, bookkeeper, attr):
        assert attr not in self.lookups_where
        from pypy.objspace.std import typeobject
        cached = "cached_where_%s" % attr
        clsdef = bookkeeper.getuniqueclassdef(typeobject.W_TypeObject)
        classdesc = clsdef.classdesc
        classdesc.immutable_fields.add(cached)
        classdesc.classdict[cached] = Constant((None, None))
        clsdef.add_source_for_attribute(cached, classdesc)
        for w_t in self.types_w:
            if not (w_t.is_heaptype() or w_t.is_cpytype()):
                setattr(w_t, cached, w_t._lookup_where(attr))
                source = InstanceSource(bookkeeper, w_t)
                clsdef.add_source_for_attribute(cached, source)
        self.lookups_where.add(attr)

    def specialize__lookup(self, funcdesc, args_s):
        # approach: add to every subclass of W_Root that does not represent a
        # user-defined class an attribute "_static_lookup_cache" which is an
        # instance of StaticLookupCache. for user-defined subclasses of
        # concrete subclasses of W_Root, _static_lookup_cache is None.
        # then add attributes cached_<special_method_name> to the instances of
        # StaticLookupCache, storing the result of looking up a special method
        # from the type
        s_space, s_obj, s_name = args_s
        if s_name.is_constant():
            attr = s_name.const
            def builder(translator, func):
                #print "LOOKUP", attr
                self.consider_lookup(funcdesc.bookkeeper, attr)
                d = {'__name__': '<ann_override_lookup>'}
                exec CACHED_LOOKUP % {'attr': attr} in d
                return translator.buildflowgraph(d['lookup_'+attr])
            return funcdesc.cachedgraph(attr, builder=builder)
        else:
            return funcdesc.cachedgraph(None) # don't specialize

    def specialize__lookup_in_type_where(self, funcdesc, args_s):
        # lookup_in_type_where works differently, we have a w_type, cache the
        # results on that for non-heaptypes
        s_space, s_type, s_name = args_s
        if s_name.is_constant():
            attr = s_name.const
            def builder(translator, func):
                #print "LOOKUP_IN_TYPE_WHERE", attr
                self.consider_lookup_in_type_where(funcdesc.bookkeeper, attr)
                d = {'__name__': '<ann_override_lookup>'}
                exec CACHED_LOOKUP_IN_TYPE_WHERE % {'attr': attr} in d
                return translator.buildflowgraph(d['lookup_in_type_where_'+attr])
            return funcdesc.cachedgraph(attr, builder=builder)
        else:
            return funcdesc.cachedgraph(None)

    def event(self, bookkeeper, what, x):
        from pypy.objspace.std import typeobject
        from pypy.interpreter import baseobjspace
        if what == "classdef_setup" and issubclass(x.classdesc.pyobj, baseobjspace.W_Root):
            classdesc = x.classdesc
            cls = classdesc.pyobj
            if (getattr(cls, "typedef", None) is None or
                    cls.user_overridden_class):
                set_attribute(classdesc, None)
                return
            w_type = self.space.gettypeobject(cls.typedef)
            if w_type.is_heaptype() or w_type.is_cpytype():
                set_attribute(classdesc, None)
                return
            if '_static_lookup_cache' not in cls.__dict__:
                set_attribute(classdesc, StaticLookupCache())
                assert '_static_lookup_cache' in cls.__dict__
            else:
                assert 0, "should not be possible"
            self.pypytypes[cls] = w_type
            clsdef = bookkeeper.getuniqueclassdef(StaticLookupCache)
            for attr in self.lookups:
                cached = "cached_%s" % attr
                w_value = w_type._lookup(attr)
                setattr(cls._static_lookup_cache, cached, w_value)
                source = InstanceSource(bookkeeper, cls._static_lookup_cache)
                clsdef.add_source_for_attribute(cached, source)
        if isinstance(x, typeobject.W_TypeObject):
            clsdef = bookkeeper.getuniqueclassdef(typeobject.W_TypeObject)
            self.types_w.add(x)
            #print "TYPE", x
            for attr in self.lookups_where:
                if not (x.is_heaptype() or x.is_cpytype()):
                    cached = "cached_where_%s" % attr
                    setattr(x, cached, x._lookup_where(attr))
                    source = InstanceSource(bookkeeper, x)
                    clsdef.add_source_for_attribute(cached, source)
        return

class StaticLookupCache(object):
    pass

CACHED_LOOKUP = """
def lookup_%(attr)s(space, w_obj, name):
    cache = w_obj._static_lookup_cache
    if cache is None:
        w_type = space.type(w_obj)
        return w_type.lookup("%(attr)s")
    return cache.cached_%(attr)s
"""

CACHED_LOOKUP_IN_TYPE_WHERE = """
def lookup_in_type_where_%(attr)s(space, w_type, name):
    if not w_type.is_heaptype() and not w_type.is_cpytype():
        return w_type.cached_where_%(attr)s
    return w_type.lookup_where("%(attr)s")
"""



import pytest
try:
    import rply
    import z3
except ImportError:
    pytest.skip('rply or z3 not installed')

from rpython.rlib.rarithmetic import LONG_BIT, r_uint, intmask, ovfcheck, uint_mul_high

from rpython.jit.metainterp.ruleopt.codegen import *
from rpython.jit.metainterp.ruleopt.parse import *

import os
with open(os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), "real.rules")) as f:
    ALLRULES = f.read()


def test_generate_commutative_rules():
    s = """\
add_zero: int_add(x, 0)
    => x
"""
    ast = parse(s)
    patterns = list(generate_commutative_patterns(ast.rules[0].pattern))
    assert str(patterns[0]) == "int_add(x, 0)"
    assert str(patterns[1]) == "int_add(0, x)"
    assert len(patterns) == 2

    s = """\
add_reassoc_consts: int_add(int_add(x, C1), C2)
    C = C1 + C2
    => int_add(x, C)
"""
    ast = parse(s)
    patterns = list(generate_commutative_patterns(ast.rules[0].pattern))
    assert [str(p) for p in patterns] == [
        'int_add(int_add(x, C1), C2)',
        'int_add(C2, int_add(x, C1))',
        'int_add(int_add(C1, x), C2)',
        'int_add(C2, int_add(C1, x))'
    ]

def test_generate_commutative_rules_only_when_necessary():
    s = """\
or_x_x: int_or(x, x)
    => x
"""
    ast = parse(s)
    patterns = list(generate_commutative_patterns(ast.rules[0].pattern))
    assert str(patterns[0]) == "int_or(x, x)"
    assert len(patterns) == 1

def test_sort_patterns():
    s = """\
int_sub_zero: int_sub(x, 0)
    => x
int_sub_x_x: int_sub(x, x)
    => 0
int_sub_add: int_sub(int_add(x, y), y)
    => x
int_sub_zero_neg: int_sub(0, x)
    => int_neg(x)
    """
    ast = parse(s)
    rules = sort_rules(ast.rules)
    assert [r.name for r in rules] == ['int_sub_zero', 'int_sub_zero_neg', 'int_sub_add', 'int_sub_x_x']

def test_create_matcher():
    s = """\
sub_from_zero: int_sub(0, x)
    => int_neg(x)

sub_add_consts: int_sub(int_add(x, C1), C2)
    C = C2 - C1
    => int_sub(x, C)

sub_add_consts: int_sub(int_add(C1, x), C2)
    C = C2 - C1
    => int_sub(x, C)
    """
    ast = parse(s)
    matcher = create_matcher(ast.rules)
    assert isinstance(matcher, IsConstMatcher)
    assert matcher.name == "arg_0"
    assert matcher.ifyes.rules[0].name == "sub_from_zero"
    assert matcher.ifyes.bindings == {(('int_sub', 0),): 'arg_0', (('int_sub', 1),): 'arg_1', (('int_sub', 0), 'C'): 'C_arg_0'}
    assert matcher.nextmatcher is None
    ifno = matcher.ifno
    assert isinstance(ifno, OpMatcher)
    assert ifno.name == "arg_0"


def test_generate_code_many():
    codegen = Codegen()
    res = codegen.generate_code(parse(ALLRULES))
    print(res)




import os, sys
from rpython.tool.version import get_repo_version_info, _get_hg_archive_version

def test_hg_archival_version(tmpdir):
    def version_for(name, items):
        path = tmpdir.join(name)
        path.write('\n'.join(('%s: %s' % (tag,value) for tag,value in items)))
        return _get_hg_archive_version(str(path))

    assert version_for('release',
                       (('tag', 'release-123'),
                        ('tag', 'ignore-me'),
                        ('node', '000'),
                       ),
                      ) == ('release-123', '000')
    assert version_for('somebranch',
                       (('node', '000'),
                        ('branch', 'something'),
                       ),
                      ) == ('something', '000')


def test_get_repo_version_info():
    assert get_repo_version_info(None)
    assert get_repo_version_info(os.devnull) == ('?', '?')
    assert get_repo_version_info(sys.executable) == ('?', '?')



import sys
import types

# Evil implicit relative import.
from kinds import KINDS


def main(argv):
    if len(argv) == 2:
        [_, modname] = argv
        attr = None
    elif len(argv) == 3:
        [_, modname, attr] = argv
    else:
        sys.exit("Wrong number of args")
    __import__(modname)
    obj = sys.modules[modname]

    if attr is not None:
        obj = getattr(obj, attr)

    for name in dir(obj):
        if attr is None and name.startswith("_"):
            continue
        subobj = getattr(obj, name)
        if subobj is None:
            continue
        if isinstance(subobj, types.FunctionType):
            try:
                subobj()
            except NotImplementedError:
                continue
            except:
                pass
        if isinstance(subobj, types.TypeType):
            kind = KINDS["TYPE"]
        else:
            kind = KINDS["UNKNOWN"]
        print kind, ":", name

if __name__ == "__main__":
    main(sys.argv)



from rpython.flowspace.model import Constant
from rpython.rtyper.error import TyperError
from rpython.rtyper.lltypesystem.lltype import Signed, Void, Ptr
from rpython.rtyper.rlist import dum_nocheck, dum_checkidx
from rpython.rtyper.rmodel import Repr, IteratorRepr
from rpython.rtyper.rint import IntegerRepr
from rpython.tool.pairtype import pairtype


class AbstractRangeRepr(Repr):
    def __init__(self, step):
        self.step = step
        if step != 0:
            self.lowleveltype = self.RANGE
        else:
            self.lowleveltype = self.RANGEST

    def _getstep(self, v_rng, hop):
        return hop.genop(self.getfield_opname,
                [v_rng, hop.inputconst(Void, 'step')], resulttype=Signed)

    def rtype_len(self, hop):
        v_rng, = hop.inputargs(self)
        if self.step == 1:
            return hop.gendirectcall(ll_rangelen1, v_rng)
        elif self.step != 0:
            v_step = hop.inputconst(Signed, self.step)
        else:
            v_step = self._getstep(v_rng, hop)
        return hop.gendirectcall(ll_rangelen, v_rng, v_step)

class __extend__(pairtype(AbstractRangeRepr, IntegerRepr)):

    def rtype_getitem((r_rng, r_int), hop):
        if hop.has_implicit_exception(IndexError):
            spec = dum_checkidx
        else:
            spec = dum_nocheck
        v_func = hop.inputconst(Void, spec)
        v_lst, v_index = hop.inputargs(r_rng, Signed)
        if r_rng.step != 0:
            cstep = hop.inputconst(Signed, r_rng.step)
        else:
            cstep = r_rng._getstep(v_lst, hop)
        if hop.args_s[1].nonneg:
            llfn = ll_rangeitem_nonneg
        else:
            llfn = ll_rangeitem
        hop.exception_is_here()
        return hop.gendirectcall(llfn, v_func, v_lst, v_index, cstep)

# ____________________________________________________________
#
#  Low-level methods.

def _ll_rangelen(start, stop, step):
    if step > 0:
        result = (stop - start + (step - 1)) // step
    else:
        result = (start - stop - (step + 1)) // (-step)
    if result < 0:
        result = 0
    return result

def ll_rangelen(l, step):
    return _ll_rangelen(l.start, l.stop, step)

def ll_rangelen1(l):
    result = l.stop - l.start
    if result < 0:
        result = 0
    return result

def ll_rangeitem_nonneg(func, l, index, step):
    if func is dum_checkidx and index >= _ll_rangelen(l.start, l.stop, step):
        raise IndexError
    return l.start + index * step

def ll_rangeitem(func, l, index, step):
    if func is dum_checkidx:
        length = _ll_rangelen(l.start, l.stop, step)
        if index < 0:
            index += length
        if index < 0 or index >= length:
            raise IndexError
    else:
        if index < 0:
            length = _ll_rangelen(l.start, l.stop, step)
            index += length
    return l.start + index * step

# ____________________________________________________________
#
#  Irregular operations.

def rtype_builtin_range(hop):
    vstep = hop.inputconst(Signed, 1)
    if hop.nb_args == 1:
        vstart = hop.inputconst(Signed, 0)
        vstop, = hop.inputargs(Signed)
    elif hop.nb_args == 2:
        vstart, vstop = hop.inputargs(Signed, Signed)
    else:
        vstart, vstop, vstep = hop.inputargs(Signed, Signed, Signed)
        if isinstance(vstep, Constant) and vstep.value == 0:
            # not really needed, annotator catches it. Just in case...
            raise TyperError("range cannot have a const step of zero")
    if isinstance(hop.r_result, AbstractRangeRepr):
        if hop.r_result.step != 0:
            c_rng = hop.inputconst(Void, hop.r_result.RANGE)
            hop.exception_is_here()
            return hop.gendirectcall(hop.r_result.ll_newrange, c_rng, vstart, vstop)
        else:
            hop.exception_is_here()
            return hop.gendirectcall(hop.r_result.ll_newrangest, vstart, vstop, vstep)
    else:
        # cannot build a RANGE object, needs a real list
        r_list = hop.r_result
        ITEMTYPE = r_list.lowleveltype
        if isinstance(ITEMTYPE, Ptr):
            ITEMTYPE = ITEMTYPE.TO
        cLIST = hop.inputconst(Void, ITEMTYPE)
        hop.exception_is_here()
        return hop.gendirectcall(ll_range2list, cLIST, vstart, vstop, vstep)

rtype_builtin_xrange = rtype_builtin_range

def ll_range2list(LIST, start, stop, step):
    if step == 0:
        raise ValueError
    length = _ll_rangelen(start, stop, step)
    l = LIST.ll_newlist(length)
    if LIST.ITEM is not Void:
        idx = 0
        while idx < length:
            l.ll_setitem_fast(idx, start)
            start += step
            idx += 1
    return l

# ____________________________________________________________
#
#  Iteration.

class AbstractRangeIteratorRepr(IteratorRepr):
    def __init__(self, r_rng):
        self.r_rng = r_rng
        if r_rng.step != 0:
            self.lowleveltype = r_rng.RANGEITER
        else:
            self.lowleveltype = r_rng.RANGESTITER

    def newiter(self, hop):
        v_rng, = hop.inputargs(self.r_rng)
        citerptr = hop.inputconst(Void, self.lowleveltype)
        return hop.gendirectcall(self.ll_rangeiter, citerptr, v_rng)

    def rtype_next(self, hop):
        v_iter, = hop.inputargs(self)
        args = hop.inputconst(Signed, self.r_rng.step),
        if self.r_rng.step > 0:
            llfn = ll_rangenext_up
        elif self.r_rng.step < 0:
            llfn = ll_rangenext_down
        else:
            llfn = ll_rangenext_updown
            args = ()
        hop.has_implicit_exception(StopIteration) # record that we know about it
        hop.exception_is_here()
        return hop.gendirectcall(llfn, v_iter, *args)

def ll_rangenext_up(iter, step):
    next = iter.next
    if next >= iter.stop:
        raise StopIteration
    iter.next = next + step
    return next

def ll_rangenext_down(iter, step):
    next = iter.next
    if next <= iter.stop:
        raise StopIteration
    iter.next = next + step
    return next

def ll_rangenext_updown(iter):
    step = iter.step
    if step > 0:
        return ll_rangenext_up(iter, step)
    else:
        return ll_rangenext_down(iter, step)

# ____________________________________________________________
#
# Support for enumerate().

class EnumerateIteratorRepr(IteratorRepr):
    def __init__(self, r_baseiter, const_startindex):
        self.r_baseiter = r_baseiter
        self.lowleveltype = r_baseiter.lowleveltype
        # only supports for now enumerate() on sequence types whose iterators
        # have a method ll_getnextindex.  It could be added for most
        # iterator types, but it's a bit messy for no clear benefit.
        if not hasattr(r_baseiter, 'll_getnextindex'):
            raise TyperError("not implemented for now: enumerate(x) where x "
                             "is not a regular list (got %r)" % (r_baseiter,))
        self.ll_getnextindex = r_baseiter.ll_getnextindex
        self.const_startindex = const_startindex

    def rtype_next(self, hop):
        v_enumerate, = hop.inputargs(self)
        v_index = hop.gendirectcall(self.ll_getnextindex, v_enumerate)
        if self.const_startindex is not None:
            v_index = hop.llops.genop(
                "int_add",
                [v_index, hop.llops.genconst(self.const_startindex)],
                resulttype=v_index.concretetype)
        hop2 = hop.copy()
        hop2.args_r = [self.r_baseiter]
        r_item_src = self.r_baseiter.external_item_repr
        r_item_dst = hop.r_result.items_r[1]
        v_item = self.r_baseiter.rtype_next(hop2)
        v_item = hop.llops.convertvar(v_item, r_item_src, r_item_dst)
        return hop.r_result.newtuple(hop.llops, hop.r_result,
                                     [v_index, v_item])

def rtype_builtin_enumerate(hop):
    hop.exception_cannot_occur()
    hop2 = hop.copy()
    hop2.args_r = [hop.args_r[0]]
    hop2.args_v = [hop.args_v[0]]
    hop2.args_s = [hop.args_s[0]]
    return hop.r_result.r_baseiter.newiter(hop2)




from pypy.interpreter.mixedmodule import MixedModule
import os

_WIN = os.name == "nt"

class Module(MixedModule):
    applevel_name = 'time'

    interpleveldefs = {
        'time': 'interp_time.time',
        'clock': 'interp_time.clock',
        'ctime': 'interp_time.ctime',
        'asctime': 'interp_time.asctime',
        'gmtime': 'interp_time.gmtime',
        'localtime': 'interp_time.localtime',
        'mktime': 'interp_time.mktime',
        'strftime': 'interp_time.strftime',
        'sleep' : 'interp_time.sleep',
    }

    if os.name == "posix":
        interpleveldefs['tzset'] = 'interp_time.tzset'

    appleveldefs = {
        'struct_time': 'app_time.struct_time',
        '__doc__': 'app_time.__doc__',
        'strptime': 'app_time.strptime',
    }

    def startup(self, space):
        if _WIN:
            from pypy.module.time.interp_time import State
            space.fromcache(State).startup(space)

        # this machinery is needed to expose constants
        # that have to be initialized one time only
        from pypy.module.time import interp_time

        interp_time._init_timezone(space)
        interp_time._init_accept2dyear(space)




from pypy.module.cpyext.test.test_cpyext import AppTestCpythonExtensionBase
from pypy.module.cpyext.test.test_api import BaseApiTest
from rpython.rtyper.lltypesystem import rffi

class AppTestSysModule(AppTestCpythonExtensionBase):
    def test_sysmodule(self):
        module = self.import_extension('foo', [
            ("get", "METH_VARARGS",
             """
                 char *name = PyString_AsString(PyTuple_GetItem(args, 0));
                 PyObject *retval = PySys_GetObject(name);
                 return PyBool_FromLong(retval != NULL);
             """)])
        assert module.get("excepthook")
        assert not module.get("spam_spam_spam")

    def test_writestdout(self):
        module = self.import_extension('foo', [
            ("writestdout", "METH_NOARGS",
             """
                 PySys_WriteStdout("format: %d\\n", 42);
                 Py_RETURN_NONE;
             """)])
        import sys, StringIO
        prev = sys.stdout
        sys.stdout = StringIO.StringIO()
        try:
            module.writestdout()
            assert sys.stdout.getvalue() == "format: 42\n"
        finally:
            sys.stdout = prev

class TestSysModule(BaseApiTest):
    def test_sysmodule(self, space, api):
        buf = rffi.str2charp("last_tb")
        api.PySys_SetObject(buf, space.wrap(1))
        rffi.free_charp(buf)
        assert space.unwrap(space.sys.get("last_tb")) == 1
